@@ -398,7 +398,7 @@ char * str_replace_all(apr_pool_t *pool, const char *string,
 char * dbfilename(apr_pool_t * pool, char * template,
                   mapcache_tileset * tileset, mapcache_grid * grid,
                   apr_array_header_t * dimensions, apr_hash_t * fmt, int z,
-                  int dbx, int dby, int xcount, int ycount)
+                  int dbx, int dby, int xcount, int ycount, int top)
 {
   int tilx = dbx * xcount;
   int tily = dby * ycount;
@@ -469,6 +469,26 @@ char * dbfilename(apr_pool_t * pool, char * template,
         apr_psprintf(pool, curfmt, dby));
   }
 
+  // Z top
+  if (top > 0) {
+    char * curfmt;
+    curfmt = apr_hash_get(fmt, "top", APR_HASH_KEY_STRING);
+    path = str_replace_all(pool, path, "{top}",
+        apr_psprintf(pool, curfmt, top));
+    curfmt = apr_hash_get(fmt, "top_x", APR_HASH_KEY_STRING);
+    path = str_replace_all(pool, path, "{top_x}",
+        apr_psprintf(pool, curfmt, dbx));
+    curfmt = apr_hash_get(fmt, "inv_top_x", APR_HASH_KEY_STRING);
+    path = str_replace_all(pool, path, "{inv_top_x}",
+        apr_psprintf(pool, curfmt, dbx));
+    curfmt = apr_hash_get(fmt, "top_y", APR_HASH_KEY_STRING);
+    path = str_replace_all(pool, path, "{top_y}",
+        apr_psprintf(pool, curfmt, dby));
+    curfmt = apr_hash_get(fmt, "inv_top_y", APR_HASH_KEY_STRING);
+    path = str_replace_all(pool, path, "{inv_top_y}",
+        apr_psprintf(pool, curfmt, dby));
+  }
+
   return path;
 }
 
@@ -659,7 +679,7 @@ int main(int argc, char * argv[])
     int minzoom, maxzoom;
     char * dbfile;
     apr_hash_t * formats;
-    int xcount, ycount;
+    int xcount, ycount, top;
   } *cache;
   apr_array_header_t * caches = NULL;
   int i, ix, iy, iz;
@@ -694,7 +714,7 @@ int main(int argc, char * argv[])
   mapcache_context_init(&ctx);
   ctx.config = mapcache_configuration_create(ctx.pool);
   ctx.log = mapcache_log;
-  mapcache_connection_pool_create(&ctx.connection_pool, ctx.pool);
+  mapcache_connection_pool_create(ctx.config, &ctx.connection_pool, ctx.pool);
 
 
   /////////////////////////////////////////////////////////////////////////////
@@ -1067,6 +1087,11 @@ int main(int argc, char * argv[])
     apr_hash_set(c->formats, "div_y",     APR_HASH_KEY_STRING, "(not set)");
     apr_hash_set(c->formats, "inv_div_x", APR_HASH_KEY_STRING, "(not set)");
     apr_hash_set(c->formats, "inv_div_y", APR_HASH_KEY_STRING, "(not set)");
+    apr_hash_set(c->formats, "top",       APR_HASH_KEY_STRING, "(not set)");
+    apr_hash_set(c->formats, "top_x",     APR_HASH_KEY_STRING, "(not set)");
+    apr_hash_set(c->formats, "top_y",     APR_HASH_KEY_STRING, "(not set)");
+    apr_hash_set(c->formats, "inv_top_x", APR_HASH_KEY_STRING, "(not set)");
+    apr_hash_set(c->formats, "inv_top_y", APR_HASH_KEY_STRING, "(not set)");
     for (hi = apr_hash_first(ctx.pool, c->formats)
         ; hi
         ; hi = apr_hash_next(hi))
@@ -1089,6 +1114,13 @@ int main(int argc, char * argv[])
     text = NULL;
     if (node) text = node->txt;
     if (text) c->ycount = (int)strtol(text, NULL, 10);
+
+    // Read top
+    c->top = -1;
+    node = ezxml_child(c->node, "top");
+    text = NULL;
+    if (node) text = node->txt;
+    if (text) c->top = (int)strtol(text, NULL, 10);
   }
 
 
@@ -1237,6 +1269,7 @@ int main(int argc, char * argv[])
     mapcache_extent_i db_region_bbox;
     int dbx_has_inv = FALSE;
     int dby_has_inv = FALSE;
+    int file_zoom_level;
 
     // Select cache according to zoom level
     for ( cid=0 ; cid < caches->nelts ; cid++ ) {
@@ -1260,16 +1293,45 @@ int main(int argc, char * argv[])
 
     // Compute region bounding box expressed in tiles and in DB files for the
     // current zoom level
-    mapcache_grid_get_xy(&ctx, grid, region_bbox.minx, region_bbox.miny, iz,
-        &(til_region_bbox.minx), &(til_region_bbox.miny));
-    mapcache_grid_get_xy(&ctx, grid, region_bbox.maxx, region_bbox.maxy, iz,
-        &(til_region_bbox.maxx), &(til_region_bbox.maxy));
+    file_zoom_level = iz;
+    if (cache->top > 0) file_zoom_level = cache->top;
+    mapcache_grid_get_xy(&ctx, grid, region_bbox.minx, region_bbox.miny,
+        file_zoom_level, &(til_region_bbox.minx), &(til_region_bbox.miny));
+    mapcache_grid_get_xy(&ctx, grid, region_bbox.maxx, region_bbox.maxy,
+        file_zoom_level, &(til_region_bbox.maxx), &(til_region_bbox.maxy));
+    if (til_region_bbox.minx > til_region_bbox.maxx) {
+      int swap = til_region_bbox.maxx;
+      til_region_bbox.maxx = til_region_bbox.minx;
+      til_region_bbox.minx = swap;
+    }
+    if (til_region_bbox.miny > til_region_bbox.maxy) {
+      int swap = til_region_bbox.maxy;
+      til_region_bbox.maxy = til_region_bbox.miny;
+      til_region_bbox.miny = swap;
+    }
 
     dbx_has_inv = strstr(cache->dbfile,"{inv_x}")
-      || strstr(cache->dbfile,"{inv_div_x}");
+      || strstr(cache->dbfile,"{inv_div_x}")
+      || strstr(cache->dbfile,"{inv_top_x}");
     dby_has_inv = strstr(cache->dbfile,"{inv_y}")
-      || strstr(cache->dbfile,"{inv_div_y}");
-    if ((cache->xcount > 0) && (cache->ycount > 0)) {
+      || strstr(cache->dbfile,"{inv_div_y}")
+      || strstr(cache->dbfile,"{inv_top_y}");
+    if (cache->top > 0) {
+      if (dbx_has_inv) {
+        db_region_bbox.minx = grid->levels[cache->top]->maxx-1 - til_region_bbox.maxx;
+        db_region_bbox.maxx = grid->levels[cache->top]->maxx-1 - til_region_bbox.minx;
+      } else {
+        db_region_bbox.minx = til_region_bbox.minx;
+        db_region_bbox.maxx = til_region_bbox.maxx;
+      }
+      if (dby_has_inv) {
+        db_region_bbox.miny = grid->levels[cache->top]->maxy-1 - til_region_bbox.maxy;
+        db_region_bbox.maxy = grid->levels[cache->top]->maxy-1 - til_region_bbox.miny;
+      } else {
+        db_region_bbox.miny = til_region_bbox.miny;
+        db_region_bbox.maxy = til_region_bbox.maxy;
+      }
+    } else if ((cache->xcount > 0) && (cache->ycount > 0)) {
       if (dbx_has_inv) {
         int inv_minx = grid->levels[iz]->maxx - til_region_bbox.minx;
         int inv_maxx = grid->levels[iz]->maxx - til_region_bbox.maxx;
@@ -1340,7 +1402,7 @@ int main(int argc, char * argv[])
         // Retrieve DB file name and check for its existence (read access)
         file_name = dbfilename(ctx.pool, cache->dbfile, tileset, grid,
             dimensions, cache->formats, iz, ix, iy, cache->xcount,
-            cache->ycount);
+            cache->ycount, cache->top);
 
         // Unless this has already been done on this file,
         // Retrieve file size and count cached tiles regardless the region of
@@ -1379,7 +1441,22 @@ int main(int argc, char * argv[])
         }
 
         // Compute file bounding box expressed in tiles
-        if ((cache->xcount > 0) && (cache->ycount > 0)) {
+        if (cache->top > 0) {
+          if (dbx_has_inv) {
+            til_file_bbox.minx = grid->levels[cache->top]->maxx-1 - ix;
+            til_file_bbox.maxx = til_file_bbox.minx;
+          } else {
+            til_file_bbox.minx = ix;
+            til_file_bbox.maxx = til_file_bbox.minx;
+          }
+          if (dby_has_inv) {
+            til_file_bbox.miny = grid->levels[cache->top]->maxy-1 - iy;
+            til_file_bbox.maxy = til_file_bbox.miny;
+          } else {
+            til_file_bbox.miny = iy;
+            til_file_bbox.maxy = til_file_bbox.miny;
+          }
+        } else if ((cache->xcount > 0) && (cache->ycount > 0)) {
           if (dbx_has_inv) {
             til_file_bbox.maxx = grid->levels[iz]->maxx-1 - ix * cache->xcount;
             til_file_bbox.minx = til_file_bbox.maxx + cache->xcount + 1;
@@ -1412,15 +1489,25 @@ int main(int argc, char * argv[])
         // Compute file bounding box expressed in grid units for the current
         // zoom level
         mapcache_grid_get_tile_extent(&ctx, grid, til_file_bbox.minx,
-            til_file_bbox.miny, iz, &temp_bbox);
+            til_file_bbox.miny, file_zoom_level, &temp_bbox);
         if (GC_HAS_ERROR(&ctx)) goto failure;
         file_bbox.minx = temp_bbox.minx;
         file_bbox.miny = temp_bbox.miny;
         mapcache_grid_get_tile_extent(&ctx, grid, til_file_bbox.maxx,
-            til_file_bbox.maxy, iz, &temp_bbox);
+            til_file_bbox.maxy, file_zoom_level, &temp_bbox);
         if (GC_HAS_ERROR(&ctx)) goto failure;
         file_bbox.maxx = temp_bbox.maxx;
         file_bbox.maxy = temp_bbox.maxy;
+        if (file_bbox.minx > file_bbox.maxx) {
+          int swap = file_bbox.maxx;
+          file_bbox.maxx = file_bbox.minx;
+          file_bbox.minx = swap;
+        }
+        if (file_bbox.miny > file_bbox.maxy) {
+          int swap = file_bbox.maxy;
+          file_bbox.maxy = file_bbox.miny;
+          file_bbox.miny = swap;
+        }
 
         // Compute part of region of interest within file bounding box
 #ifdef USE_CLIPPERS
@@ -1487,6 +1574,16 @@ int main(int argc, char * argv[])
         mapcache_grid_get_xy(&ctx, grid, region_in_file_bbox.maxx-res,
             region_in_file_bbox.maxy-res, iz, &(til_region_in_file_bbox.maxx),
             &(til_region_in_file_bbox.maxy));
+        if (til_region_in_file_bbox.minx > til_region_in_file_bbox.maxx) {
+          int swap = til_region_in_file_bbox.maxx;
+          til_region_in_file_bbox.maxx = til_region_in_file_bbox.minx;
+          til_region_in_file_bbox.minx = swap;
+        }
+        if (til_region_in_file_bbox.miny > til_region_in_file_bbox.maxy) {
+          int swap = til_region_in_file_bbox.maxy;
+          til_region_in_file_bbox.maxy = til_region_in_file_bbox.miny;
+          til_region_in_file_bbox.miny = swap;
+        }
         if ((cache->xcount > 0) && (cache->ycount > 0)) {
           if (til_region_in_file_bbox.maxx>(til_file_bbox.minx+cache->xcount-1))
           {
