@@ -327,6 +327,8 @@ typedef enum {
   ,MAPCACHE_CACHE_COMPOSITE
   ,MAPCACHE_CACHE_COUCHBASE
   ,MAPCACHE_CACHE_RIAK
+  ,MAPCACHE_CACHE_REDIS
+
 } mapcache_cache_type;
 
 /** \interface mapcache_cache
@@ -408,6 +410,11 @@ mapcache_cache* mapcache_cache_tc_create(mapcache_context *ctx);
  */
 mapcache_cache* mapcache_cache_riak_create(mapcache_context *ctx);
 
+/**
+ * \memberof mapcache_cache_redis
+ */
+mapcache_cache* mapcache_cache_redis_create(mapcache_context* ctx);
+
 /** @} */
 
 
@@ -1442,7 +1449,7 @@ MS_DLL_EXPORT char *mapcache_util_str_replace(apr_pool_t *pool, const char *stri
                                 const char *replacement );
 char *mapcache_util_dbl_replace(apr_pool_t *pool, const char *string, const char *substr,
                                 double replacement );
-char *mapcache_util_str_replace_all(apr_pool_t *pool, const char *string, const char *substr,
+MS_DLL_EXPORT char *mapcache_util_str_replace_all(apr_pool_t *pool, const char *string, const char *substr,
                                     const char *replacement );
 char *mapcache_util_dbl_replace_all(apr_pool_t *pool, const char *string, const char *substr,
                                     double replacement );
