@@ -209,7 +209,8 @@ void mapcache_tileset_get_map_tiles(mapcache_context *ctx, mapcache_tileset *til
                                     mapcache_extent *bbox, int width, int height,
                                     int *ntiles,
                                     mapcache_tile ***tiles,
-                                    mapcache_grid_link **effectively_used_grid_link)
+                                    mapcache_grid_link **effectively_used_grid_link,
+                                    apr_array_header_t *dimensions)
 {
   double resolution;
   int level;
@@ -225,6 +226,22 @@ void mapcache_tileset_get_map_tiles(mapcache_context *ctx, mapcache_tileset *til
     level = (*effectively_used_grid_link)->max_cached_zoom;
   }
 
+  // Get dimensions values for whole map if required by <wms_querybymap> configuration
+  if (dimensions)
+  {
+    int j;
+    for (j=0 ; j<dimensions->nelts ; j++)
+    {
+      mapcache_requested_dimension *rdim = APR_ARRAY_IDX(dimensions,j,mapcache_requested_dimension*);
+      mapcache_dimension *dim = rdim->dimension;
+      if (dim->wms_querybymap_minzoom != -1 && level >= dim->wms_querybymap_minzoom) {
+        rdim->cached_entries_for_value =
+          mapcache_dimension_get_entries_for_value(ctx,rdim->dimension,rdim->requested_value,
+              tileset,bbox,(*effectively_used_grid_link)->grid);
+      }
+    }
+  }
+
   mapcache_grid_get_xy(ctx,(*effectively_used_grid_link)->grid,bbox->minx,bbox->miny,level,&bl_x,&bl_y);
   mapcache_grid_get_xy(ctx,(*effectively_used_grid_link)->grid,bbox->maxx,bbox->maxy,level,&tr_x,&tr_y);
   Mx = MAPCACHE_MAX(MAPCACHE_MIN(MAPCACHE_MAX(tr_x,bl_x),(*effectively_used_grid_link)->grid_limits[level].maxx),(*effectively_used_grid_link)->grid_limits[level].minx);
@@ -567,6 +584,7 @@ mapcache_tile* mapcache_tileset_tile_create(apr_pool_t *pool, mapcache_tileset *
       mapcache_requested_dimension *rdim = apr_pcalloc(pool,sizeof(mapcache_requested_dimension));
       rdim->requested_value = dimension->default_value;
       rdim->cached_value = NULL;
+      rdim->cached_entries_for_value = NULL;
       rdim->dimension = dimension;
       APR_ARRAY_PUSH(tile->dimensions,mapcache_requested_dimension*) = rdim;
     }
@@ -617,6 +635,7 @@ mapcache_map* mapcache_tileset_map_create(apr_pool_t *pool, mapcache_tileset *ti
       mapcache_requested_dimension *rdim = apr_pcalloc(pool,sizeof(mapcache_requested_dimension));
       rdim->requested_value = dimension->default_value;
       rdim->cached_value = NULL;
+      rdim->cached_entries_for_value = NULL;
       rdim->dimension = dimension;
       APR_ARRAY_PUSH(map->dimensions,mapcache_requested_dimension*) = rdim;
     }
@@ -641,6 +660,7 @@ mapcache_feature_info* mapcache_tileset_feature_info_create(apr_pool_t *pool, ma
       mapcache_requested_dimension *rdim = apr_pcalloc(pool,sizeof(mapcache_requested_dimension));
       rdim->requested_value = dimension->default_value;
       rdim->cached_value = NULL;
+      rdim->cached_entries_for_value = NULL;
       rdim->dimension = dimension;
       APR_ARRAY_PUSH(fi->map.dimensions,mapcache_requested_dimension*) = rdim;
     }
@@ -795,11 +815,42 @@ int mapcache_tileset_tile_get_readonly(mapcache_context *ctx, mapcache_tile *til
 
 typedef struct {
   mapcache_tile *tile;
-  int cache_status;
+  int isFetched;
 } mapcache_subtile;
 
 static void mapcache_tileset_tile_get_without_subdimensions(mapcache_context *ctx, mapcache_tile *tile, int read_only);
 
+#if APR_HAS_THREADS                                                                      
+/* use a thread pool if using 1.3.12 or higher apu */                                    
+#include "apu_version.h"                                                                 
+#if (APU_MAJOR_VERSION <= 1 && APU_MINOR_VERSION <= 3)                                   
+#define USE_THREADPOOL 0                                                                 
+#include <apr_thread_proc.h>                                                             
+#else                                                                                    
+#define USE_THREADPOOL 1                                                                 
+#include <apr_thread_pool.h>                                                             
+#endif                                                                                   
+                                                                                         
+typedef struct {                                                                         
+  mapcache_subtile *subtile;                                                             
+  mapcache_tile *tile;                                                                   
+  mapcache_context *ctx;                                                                 
+} _thread_subtile;                                                                       
+static void* APR_THREAD_FUNC _thread_get_subtile(apr_thread_t *thread, void *data)       
+{                                                                                        
+  _thread_subtile * t = (_thread_subtile *)data;                                         
+  /* creates the tile from the source, takes care of metatiling */                       
+  mapcache_tileset_tile_get_without_subdimensions(t->ctx, t->subtile->tile,              
+      (t->tile->tileset->subdimension_read_only||!t->tile->tileset->source)?1:0);        
+  t->subtile->isFetched = MAPCACHE_TRUE;                                                 
+#if !USE_THREADPOOL                                                                      
+  apr_thread_exit(thread, APR_SUCCESS);                                                  
+#endif                                                                                   
+  return NULL;                                                                           
+}                                                                                        
+#endif // APR_HAS_THREADS                                                                
+                                                                                         
+
 void mapcache_tileset_tile_set_get_with_subdimensions(mapcache_context *ctx, mapcache_tile *tile) {
   apr_array_header_t *subtiles;
   mapcache_extent extent;
@@ -813,14 +864,20 @@ void mapcache_tileset_tile_set_get_with_subdimensions(mapcache_context *ctx, map
    */
   subtiles = apr_array_make(ctx->pool,1,sizeof(mapcache_subtile));
   st.tile = tile;
+  st.isFetched = MAPCACHE_FALSE;
   APR_ARRAY_PUSH(subtiles,mapcache_subtile) = st;
   mapcache_grid_get_tile_extent(ctx,tile->grid_link->grid,tile->x,tile->y,tile->z,&extent);
   if(GC_HAS_ERROR(ctx)) goto cleanup;
 
   for(i=0;i<tile->dimensions->nelts; i++) {
     mapcache_requested_dimension *rdim = APR_ARRAY_IDX(tile->dimensions,i,mapcache_requested_dimension*);
-    apr_array_header_t *single_subdimension = mapcache_dimension_get_entries_for_value(ctx,rdim->dimension,rdim->requested_value,
-                                                                                     tile->tileset, &extent, tile->grid_link->grid);
+    apr_array_header_t *single_subdimension;
+    if (rdim->cached_entries_for_value) {
+      single_subdimension = rdim->cached_entries_for_value;
+    } else {
+      single_subdimension = mapcache_dimension_get_entries_for_value(ctx,rdim->dimension,rdim->requested_value,
+          tile->tileset, &extent, tile->grid_link->grid);
+    }
     if(GC_HAS_ERROR(ctx)) /* invalid dimension given */
       goto cleanup;
 #ifdef DEBUG
@@ -856,6 +913,7 @@ void mapcache_tileset_tile_set_get_with_subdimensions(mapcache_context *ctx, map
         /* clone the existing subtiles if we have more than one sub-dimension to assemble for the the current dimension */
         for(k=1;k<single_subdimension->nelts;k++) {
           st.tile = mapcache_tileset_tile_clone(ctx->pool,APR_ARRAY_IDX(subtiles,j,mapcache_subtile).tile);
+          st.isFetched = MAPCACHE_FALSE;
           APR_ARRAY_PUSH(subtiles,mapcache_subtile)=st;
         }
       }
@@ -872,10 +930,51 @@ void mapcache_tileset_tile_set_get_with_subdimensions(mapcache_context *ctx, map
 
   /* our subtiles array now contains a list of tiles with subdimensions split up, we now need to fetch them from the cache */
   /* note that subtiles[0].tile == tile */
+#if APR_HAS_THREADS                                                                                               
+  if (tile->tileset->assembly_threaded_fetching_maxzoom != -1
+      && tile->z <= tile->tileset->assembly_threaded_fetching_maxzoom) {
+    apr_thread_t **threads;                                                                                       
+    apr_threadattr_t *thread_attrs;                                                                               
+    int nthreads;                                                                                                 
+    _thread_subtile * thread_subtiles;                                                                            
+
+    nthreads = 0;                                                                                                 
+    apr_threadattr_create(&thread_attrs, ctx->pool);                                                              
+    thread_subtiles = (_thread_subtile*)apr_pcalloc(ctx->pool,subtiles->nelts*sizeof(_thread_subtile));           
+    threads = (apr_thread_t**)apr_pcalloc(ctx->pool,subtiles->nelts*sizeof(apr_thread_t*));                       
+    for(i=subtiles->nelts-1; i>=0; i--) {                                                                         
+      int rv;                                                                                                     
+      thread_subtiles[i].subtile = &(APR_ARRAY_IDX(subtiles,i,mapcache_subtile));                                 
+      thread_subtiles[i].tile = tile;                                                                             
+      thread_subtiles[i].ctx = ctx->clone(ctx);                                                                   
+      rv = apr_thread_create(&threads[i], thread_attrs, _thread_get_subtile,                                      
+          (void*)&(thread_subtiles[i]), thread_subtiles[i].ctx->pool);                                            
+      if (rv != APR_SUCCESS) {                                                                                    
+        // In case of failure creating a thread, the subtile will be fetched in the main loop below
+        break;                                                                             
+      }                                                                                    
+      nthreads++;                                                                          
+    }                                                                                      
+    for(i=0 ; i<nthreads ; i++) {                                                          
+      int rv;                                                                              
+      apr_thread_join(&rv, threads[i]);                                                    
+      if(rv != APR_SUCCESS) {                                                              
+        ctx->set_error(ctx,500, "thread %d of %d failed on exit\n",i,nthreads);            
+      }                                                                                    
+      if(GC_HAS_ERROR(thread_subtiles[i].ctx)) {                                           
+        /* transfer error message from child thread to main context */                     
+        ctx->set_error(ctx,thread_subtiles[i].ctx->get_error(thread_subtiles[i].ctx),      
+            thread_subtiles[i].ctx->get_error_message(thread_subtiles[i].ctx));            
+      }                                                                                    
+    }                                                                                      
+  }
+#endif // APR_HAS_THREADS
 
   for(i=subtiles->nelts-1; i>=0; i--) {
     mapcache_tile *subtile = APR_ARRAY_IDX(subtiles,i,mapcache_subtile).tile;
-    mapcache_tileset_tile_get_without_subdimensions(ctx, subtile, (tile->tileset->subdimension_read_only||!tile->tileset->source)?1:0); /* creates the tile from the source, takes care of metatiling */
+    if (!APR_ARRAY_IDX(subtiles,i,mapcache_subtile).isFetched) {
+      mapcache_tileset_tile_get_without_subdimensions(ctx, subtile, (tile->tileset->subdimension_read_only||!tile->tileset->source)?1:0); /* creates the tile from the source, takes care of metatiling */
+    }
     if(GC_HAS_ERROR(ctx))
       goto cleanup;
     if(!subtile->nodata) {
@@ -903,8 +1002,8 @@ void mapcache_tileset_tile_set_get_with_subdimensions(mapcache_context *ctx, map
         if(GC_HAS_ERROR(ctx))
           goto cleanup;
       }
-      if((subtile->encoded_data && mapcache_imageio_header_sniff(ctx,subtile->encoded_data) == GC_JPEG)||
-         (subtile->raw_image && subtile->raw_image->has_alpha == MC_ALPHA_NO)) {
+      if ((mapcache_imageio_alpha_sniff(ctx,subtile->encoded_data) == MC_ALPHA_NO) ||
+          (subtile->raw_image && subtile->raw_image->has_alpha == MC_ALPHA_NO)) {
         /* the returned image is fully opaque, we don't need to get/decode/merge any further subtiles */
         if(assembled_image)
           assembled_image->has_alpha = MC_ALPHA_NO;
