@@ -80,11 +80,13 @@ int nprocesses=0;
 int quiet = 0;
 int verbose = 0;
 int force = 0;
+int non_interactive = 0;
 int sig_int_received = 0;
 int error_detected = 0;
 double percent_failed_allowed = 1.0;
 int n_metatiles_tot = 0;
 int n_nodata_tot = 0;
+int n_skipped_tot = 0;
 int rate_limit = 0;
 FILE *failed_log = NULL, *retry_log = NULL;
 #define FAIL_BACKLOG_COUNT 1000
@@ -127,6 +129,7 @@ struct seed_status {
   s_status status;
   int x,y,z;
   int nodata;
+  int skipped;
   char *msg;
 };
 
@@ -241,6 +244,7 @@ static const apr_getopt_option_t seed_options[] = {
   { "ogr-datasource", 'd', TRUE, "ogr datasource to get features from"},
 #endif
   { "dimension", 'D', TRUE, "set the value of a dimension (format DIMENSIONNAME=VALUE). Can be used multiple times for multiple dimensions" },
+  { "header", 'H', TRUE, "set the value of a header to forward to back-end servers (format HEADERNAME=VALUE). Can be used multiple times for multiple headers" },
   { "extent", 'e', TRUE, "extent to seed, format: minx,miny,maxx,maxy" },
   { "force", 'f', FALSE, "force tile recreation even if it already exists" },
   { "grid", 'g', TRUE, "grid to seed" },
@@ -263,6 +267,7 @@ static const apr_getopt_option_t seed_options[] = {
 #endif
   { "tileset", 't', TRUE, "tileset to seed" },
   { "verbose", 'v', FALSE, "show debug log messages" },
+  { "non-interactive", 'b', FALSE, "print progress messages on new lines"},
 #ifdef USE_CLIPPERS
   { "ogr-where", 'w', TRUE, "filter to apply on layer features"},
 #endif
@@ -496,8 +501,23 @@ void cmd_recurse(mapcache_context *cmd_ctx, mapcache_tile *tile)
     if(rate_limit > 0)
       rate_limit_sleep();
     push_queue(cmd);
+  } else if (action == MAPCACHE_CMD_SKIP) {
+    apr_status_t ret;
+    struct seed_status *st = calloc(1,sizeof(struct seed_status));
+    int retries=0;
+    st->x=tile->x;
+    st->y=tile->y;
+    st->z=tile->z;
+    st->nodata = 0;
+    st->skipped = 1;
+    st->status = MAPCACHE_STATUS_OK;
+    ret = apr_queue_push(log_queue,(void*)st);
+    while( ret == APR_EINTR && retries < 10) {
+      retries++;
+      ret = apr_queue_push(log_queue,(void*)st);
+    }
   }
-  
+
   if(action == MAPCACHE_CMD_STOP_RECURSION)
     return;
 
@@ -619,6 +639,21 @@ void feed_worker()
         if(rate_limit > 0)
           rate_limit_sleep();
         push_queue(cmd);
+      } else if (action == MAPCACHE_CMD_SKIP) {
+        apr_status_t ret;
+        struct seed_status *st = calloc(1,sizeof(struct seed_status));
+        int retries=0;
+        st->x=tile->x;
+        st->y=tile->y;
+        st->z=tile->z;
+        st->nodata = 0;
+        st->skipped = 1;
+        st->status = MAPCACHE_STATUS_OK;
+        ret = apr_queue_push(log_queue,(void*)st);
+        while( ret == APR_EINTR && retries < 10) {
+          retries++;
+          ret = apr_queue_push(log_queue,(void*)st);
+        }
       }
 
       //compute next x,y,z
@@ -725,6 +760,7 @@ void seed_worker()
       st->y=tile->y;
       st->z=tile->z;
       st->nodata = tile->nodata;
+      st->skipped = 0;
       if(seed_ctx.get_error(&seed_ctx)) {
         st->status = MAPCACHE_STATUS_FAIL;
         st->msg = strdup(seed_ctx.get_error_message(&seed_ctx));
@@ -793,7 +829,11 @@ static void* APR_THREAD_FUNC log_thread_fn(apr_thread_t *thread, void *data) {
       return NULL;
     if(st->status == MAPCACHE_STATUS_OK) {
       failed[cur]=0;
-      n_metatiles_tot++;
+      if (st->skipped) {
+        n_skipped_tot++;
+      } else {
+        n_metatiles_tot++;
+      }
       if(st->nodata) {
         n_nodata_tot++;
       }
@@ -802,9 +842,16 @@ static void* APR_THREAD_FUNC log_thread_fn(apr_thread_t *thread, void *data) {
         mapcache_gettimeofday(&now,NULL);
         now_time = now.tv_sec + now.tv_usec / 1000000.0;
         if((now_time - last_time) > 1.0) {
-          printf("                                                                                               \r");
-          printf("seeded %d tiles, now at z%d x%d y%d\r",n_metatiles_tot*tileset->metasize_x*tileset->metasize_y, st->z,st->x,st->y);
-          fflush(stdout);
+          int seeded_count = n_metatiles_tot*tileset->metasize_x*tileset->metasize_y;
+          int skipped_count = n_skipped_tot*tileset->metasize_x*tileset->metasize_y;
+          if (non_interactive) {
+            printf("seeded %d tiles (%d skipped), now at z%d x%d y%d\n",seeded_count,skipped_count,st->z,st->x,st->y);
+          } else {
+            printf("                                                                                               \r");
+            printf("seeded %d tiles (%d skipped), now at z%d x%d y%d\r",seeded_count,skipped_count,st->z,st->x,st->y);
+            fflush(stdout);
+          }
+
           last_time = now_time;
         }
       }
@@ -897,6 +944,30 @@ int usage(const char *progname, char *msg, ...)
   return 1;
 }
 
+static int parseKeyValuePairs(mapcache_context ctx, apr_table_t* argtable, const char* optarg)
+{
+    char* argkey = NULL, * argvalue = NULL, * key, * last, * optargcpy = NULL;
+    int keyidx;
+
+    optargcpy = apr_pstrdup(ctx.pool, optarg);
+    keyidx = 0;
+    for (key = apr_strtok(optargcpy, "=", &last); key != NULL;
+        key = apr_strtok(NULL, "=", &last)) {
+        if (keyidx == 0) {
+            argkey = key;
+        }
+        else {
+            argvalue = key;
+        }
+        keyidx++;
+    }
+    if (keyidx != 2 || !argkey || !argvalue || !*argkey || !*argvalue) {
+        return -1;
+    } else {
+        apr_table_set(argtable, argkey, argvalue);
+        return 0;
+    }
+}
 static int isPowerOfTwo(int x)
 {
   return (x & (x - 1)) == 0;
@@ -920,8 +991,7 @@ int main(int argc, const char **argv)
   const char *old = NULL;
   const char *optarg;
   apr_table_t *argdimensions;
-  char *dimkey=NULL, *dimvalue=NULL,*key, *last, *optargcpy=NULL;
-  int keyidx;
+  apr_table_t* argheaders;
   int *metasizes = NULL;//[2];
   int metax=-1,metay=-1;
   double *extent_array = NULL;
@@ -947,13 +1017,16 @@ int main(int argc, const char **argv)
 
   mapcache_gettimeofday(&starttime,NULL);
   argdimensions = apr_table_make(ctx.pool,3);
+  argheaders = apr_table_make(ctx.pool, 3);
 
+  ctx.headers_in = argheaders;
 
   /* parse the all options based on opt_option[] */
   while ((rv = apr_getopt_long(opt, seed_options, &optch, &optarg)) == APR_SUCCESS) {
     switch (optch) {
       case 'h':
-        return usage(argv[0],NULL);
+        usage(argv[0],NULL);
+        return 0;
         break;
       case 'f':
         force = 1;
@@ -964,6 +1037,9 @@ int main(int argc, const char **argv)
       case 'v':
         verbose = 1;
         break;
+      case 'b':
+        non_interactive = 1;
+        break;
       case 'c':
         configfile = optarg;
         break;
@@ -1064,22 +1140,15 @@ int main(int argc, const char **argv)
         old = optarg;
         break;
       case 'D':
-        optargcpy = apr_pstrdup(ctx.pool,optarg);
-        keyidx = 0;
-        for (key = apr_strtok(optargcpy, "=", &last); key != NULL;
-             key = apr_strtok(NULL, "=", &last)) {
-          if(keyidx == 0) {
-            dimkey = key;
-          } else {
-            dimvalue = key;
-          }
-          keyidx++;
+        if(parseKeyValuePairs(ctx, argdimensions, optarg) < 0) {
+            return usage(argv[0], "failed to parse dimension, expecting DIMNAME=DIMVALUE");
         }
-        if(keyidx!=2 || !dimkey || !dimvalue || !*dimkey || !*dimvalue) {
-          return usage(argv[0], "failed to parse dimension, expecting DIMNAME=DIMVALUE");
-        }
-        apr_table_set(argdimensions,dimkey,dimvalue);
         break;
+      case 'H':
+          if (parseKeyValuePairs(ctx, argheaders, optarg) < 0) {
+              return usage(argv[0], "failed to parse headers, expecting HEADERNAME=VALUE");
+          }
+          break;
       case SEEDER_OPT_THREAD_DELAY:
         thread_delay = strtod(optarg, NULL);
         if(thread_delay < 0.0 )
@@ -1409,7 +1478,7 @@ int main(int argc, const char **argv)
   if(nthreads >= 1 && nprocesses >= 1) {
     return usage(argv[0],"cannot set both nthreads and nprocesses");
   }
-  
+
   {
   /* start the logging thread */
     //create the queue where the seeding statuses will be put
@@ -1420,7 +1489,7 @@ int main(int argc, const char **argv)
     apr_threadattr_create(&log_thread_attrs, ctx.pool);
     apr_thread_create(&log_thread, log_thread_attrs, log_thread_fn, NULL, ctx.pool);
   }
-  
+
   if(nprocesses > 1) {
 #ifdef USE_FORK
     key_t key;
